
showBackgroundChecked = true;
openInspectUIChecked = true;

frameBorderSize = 10;
mouseFacing = 0;
cursorX = 0;
cursorY = 0;

local backFile = "Interface\\Glues\\LoadingScreens\\LoadScreenZulGurub"
local backNone = "Interface\Tooltips\UI-Tooltip-Background";
local edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border";
local backdrop = {
    bgFile = backFile,
    edgeFile = edgeFile,
    tile = false, tileSize = 16, edgeSize = 16,
    insets = { left = 4, right = 3, top = 4, bottom = 3 }
}

-----------------------
-- KEYBOARD BINDINGS
-----------------------
BINDING_HEADER_MODELVIEWER = "Model Viewer"
BINDING_NAME_MODELVIEWER_VIEWMODEL = "View Model"

function ModelViewerMainFrame_OnLoad(self)
    ModelViewerMainFrame:RegisterEvent("VARIABLES_LOADED");
    ModelViewerMainFrame:RegisterEvent("PLAYER_TARGET_CHANGED");

    ModelViewerMainFrame:RegisterForDrag("LeftButton", "RightButton");

    ModelViewerMainFrame:SetScript("OnEvent", function(self, event, ...) ModelViewerMainFrame_OnEvent(self, event, ...); end);
    ModelViewerMainFrame:SetScript("OnUpdate", function(self, event, ...) ModelViewerMainFrame_OnUpdate(self, event, ...); end);
    ModelViewerMainFrame:SetScript("OnMouseUp", function(self, button) ModelViewerMainFrame_OnMouseUp(self, button); end);
    ModelViewerMainFrame:SetScript("OnMouseDown", function(self, button) ModelViewerMainFrame_OnMouseDown(self, button); end);
    ModelViewerMainFrame:SetScript("OnDragStart", function() ModelViewerMainFrame_OnDragStart(); end);
    ModelViewerMainFrame:SetScript("OnDragStop", function() ModelViewerMainFrame_OnDragStop(); end);

    tinsert(UISpecialFrames, "ModelViewerMainFrame"); -- Hide by ESC Key.
    tinsert(UISpecialFrames, "ModelViewerConfigFrame"); -- Hide by ESC Key.
end

function ModelViewerMainFrame_OnEvent(self, event, ...)
    if (event == "VARIABLES_LOADED") then
        ModelViewerMainFrame_OnVariablesLoaded();
    elseif (event == "PLAYER_TARGET_CHANGED") then
        ModelViewerMainFrame_SetUnit();
    end
end

function ModelViewerMainFrame_SetUnit()    
    unitid = nil;
    if (UnitExists("target")) then
        unitid = "target";
        ModelViewerMainFrame_PlayerModel:SetUnit(unitid);

        if ModelViewerMainFrame_PlayerModel:IsVisible() then
            UnitInfoLayer_Show();
            BlizzardInspectUI_Show();
        end
    else 
        ModelViewerMainFrame_Hide();
    end
end

function ModelViewerFrame_ViewModel_OnPress()
    if (ModelViewerMainFrame:IsVisible()) then
    --if (DressUpFrame:IsVisible()) then
        ModelViewerMainFrame_Hide();
    else
        if (unitid == nil) then
            unitid = "player";
            ModelViewerMainFrame_PlayerModel:SetUnit(unitid);
        end
        ModelViewerMainFrame_Show();
    end
end

function ModelViewerMainFrame_OnMouseDown(self, button)
    cursorX, cursorY = GetCursorPosition();

    if (button == "LeftButton") then
        mouseFacing = 1; -- Model Rotation Start.
    end
end

function ModelViewerMainFrame_OnMouseUp(self, button)
    local cx, cy = GetCursorPosition();
    if cursorX == cx and cursorY == cy then
        ModelViewerMainFrame_Hide();        
    end
    mouseFacing = 0; -- Model Rotation End.
end

function ModelViewerMainFrame_OnDragStart()

end

function ModelViewerMainFrame_OnDragStop()
    mouseFacing = 0;    
end

function ModelViewerMainFrame_OnUpdate()
    -------------------------------
    -- Update Model Rotation.
    -------------------------------
    if (mouseFacing == 1) then
        local cx, cy = GetCursorPosition();
        ModelViewerMainFrame_PlayerModel:SetFacing(ModelViewerMainFrame_PlayerModel:GetFacing() + ((cx - cursorX) / 50));
        cursorX, cursorY = GetCursorPosition();
    end
end

function ModelViewerMainFrame_Show()
    if (unitid == nil) then
        return;
    end

    PlaySound("GAMEDIALOGOPEN", "master"); -- http://wowwiki.wikia.com/wiki/API_PlaySound    

    --backdrop.bgFile = classFileName;
    --ModelViewerMainFrame:SetBackdrop(backdrop)

    local scale = UIParent:GetEffectiveScale();
    local width = GetScreenWidth() * scale;
    local height = GetScreenHeight() * scale;

    width = width - (width / 2.7);
    height = height - (height / 8);

    local frame = ModelViewerMainFrame;
    local model = ModelViewerMainFrame_PlayerModel;

    frame:SetWidth(width);
    frame:SetHeight(height);
    frame:ClearAllPoints();
    frame:SetPoint("CENTER", "UIParent");

    if (showBackgroundChecked) then
        ModelViewerMainFrame_ShowBackground();
    else
        ModelViewerMainFrame_HideBackground();
    end
    ShowUIPanel(frame);
    --frame:SetFrameStrata(largeWinStrata);

    --------------
    -- 모델 크기
    --------------
    width = height;
    model:SetFacing(0);
    model:SetWidth(width - frameBorderSize);
    model:SetHeight(height - frameBorderSize);
    ShowUIPanel(model);
    
    UnitInfoLayer_Show();
    BlizzardInspectUI_Show();
end

function ModelViewerMainFrame_Hide()
    if ModelViewerMainFrame:IsVisible() then
        PlaySound("GAMEDIALOGCLOSE", "master"); -- http://wowwiki.wikia.com/wiki/API_PlaySound
    end

    HideUIPanel(ModelViewerMainFrame_PlayerModel);
    HideUIPanel(ModelViewerMainFrame);
    BlizzardInspectUI_Hide();
end

function UnitInfoLayer_Show()

    local name, realm = UnitName(unitid);

    if (unitid == nil or unitid == "unit" or name == nil) then

    else
        ModelViewerMainFrameStringName:Show();
        ModelViewerMainFrameStringName:SetText(name);

        local class, classFileName = UnitClass(unitid);
        local color = RAID_CLASS_COLORS[classFileName];
        ModelViewerMainFrameStringClass:Show();
        ModelViewerMainFrameStringClass:SetText(class);
        ModelViewerMainFrameStringClass:SetTextColor(color.r, color.g, color.b, 1);

        ModelViewerMainFrameStringLevel:Show();
        ModelViewerMainFrameStringLevel:SetText("Level "..UnitLevel(unitid));

        local factionGroup, factionName = UnitFactionGroup(unitid);
        if factionName == nil then
            factionName = UnitCreatureType(unitid);
        end
        if UnitIsEnemy("player", unitid) then
            ModelViewerMainFrameStringFaction:SetTextColor(1, 0, 0, 1); -- Red
        else
            ModelViewerMainFrameStringFaction:SetTextColor(0, 1, 0, 1); -- Green
        end
        ModelViewerMainFrameStringFaction:Show();
        ModelViewerMainFrameStringFaction:SetText(factionName);

        local guildName, guildRankName, guildRankIndex = GetGuildInfo(unitid);
        if guildName then
            text = guildName.." / ".. guildRankName;
            ModelViewerMainFrameStringGuild:Show();
            ModelViewerMainFrameStringGuild:SetText(text);
        else
            ModelViewerMainFrameStringGuild:Hide();
        end

        --if UnitCreatureFamily(unitid) then text = text.." / "..UnitCreatureFamily(unitid); end
        --if UnitClassification(unitid) then text = text.." / "..UnitClassification(unitid); end
    end
end

function UnitInfoLayer_Hide()
    ModelViewerMainFrameStringName:Hide();
    ModelViewerMainFrameStringClass:Hide();
    ModelViewerMainFrameStringLevel:Hide();
    ModelViewerMainFrameStringFaction:Hide();
    ModelViewerMainFrameStringGuild:Hide();
end 

----------------------------------
-- Option: Show Background
----------------------------------
function ModelViewerMainFrame_CbBackground_OnLoad()
    ModelViewerMainFrame_CbBackgroundText:SetText(" Show Background.");    
end
function ModelViewerMainFrame_CbBackground_OnClick()
    if ModelViewerMainFrame_CbBackground:GetChecked() then
        showBackgroundChecked = true;
        ModelViewerMainFrame_ShowBackground();
    else
        showBackgroundChecked = false;
        ModelViewerMainFrame_HideBackground();
    end
    ModelViewerMainFrame_SaveVariables();
end

function ModelViewerMainFrame_ShowBackground()
    local frame = ModelViewerMainFrame;
    local max = table.getn(blps);
    local key = math.random (0, max);    
    backdrop.bgFile = "Interface\\Glues\\LoadingScreens\\"..blps[key];
    backdrop.title = false;
    frame:SetBackdropColor(1, 1, 1, 0);
    frame:SetBackdrop(backdrop);
end

function ModelViewerMainFrame_HideBackground()
    local frame = ModelViewerMainFrame;
    backdrop.bgFile = backNone;
    backdrop.title = true;
    frame:SetBackdrop(backdrop);
    frame:SetBackdropColor(0, 0, 0, 0.3);
end

----------------------------------
-- Option: Opne Inspect Window
----------------------------------
function ModelViewerMainFrame_CbOpenInspectUI_OnLoad()
    ModelViewerMainFrame_CbOpenInspectUIText:SetText(" Open Inspect Window.");
end
function ModelViewerMainFrame_CbOpenInspectUI_OnClick()
    if ModelViewerMainFrame_CbOpenInspectUI:GetChecked() then
        openInspectUIChecked = true;
        BlizzardInspectUI_Show();
    else
        openInspectUIChecked = false;
        BlizzardInspectUI_Hide();
    end
    ModelViewerMainFrame_SaveVariables();
end

function BlizzardInspectUI_Show()
    if (openInspectUIChecked and CanInspect(unitid)) then
        InspectUnit(unitid);
    end
end

function BlizzardInspectUI_Hide()
    if InspectFrame:IsVisible() then
        HideUIPanel(InspectFrame);
    end    
end

function ModelViewerMainFrame_OnVariablesLoaded()
    showBackgroundChecked = ModelViewerSettings.showBackgroundChecked;
    if (showBackgroundChecked == nil) then
        showBackgroundChecked = true;
    end
    ModelViewerMainFrame_CbBackground:SetChecked(showBackgroundChecked);

    openInspectUIChecked = ModelViewerSettings.openInspectUIChecked;
    if (openInspectUIChecked == nil) then
        openInspectUIChecked = true;
    end
    ModelViewerMainFrame_CbOpenInspectUI:SetChecked(openInspectUIChecked);
end

function ModelViewerMainFrame_SaveVariables()
    ModelViewerSettings = {
        showBackgroundChecked = showBackgroundChecked,
        openInspectUIChecked = openInspectUIChecked
    }
end

-- -----------------------------------------------------------------------------------------------
-- https://us.battle.net/support/en/article/download-the-world-of-warcraft-interface-addon-kit
-- -----------------------------------------------------------------------------------------------
blps = {
"LoadScreenArgentDungeon.blp",
"LOADSCREENARGENTRAID.BLP",
"LOADSCREENAUCHINDOUN.BLP",
"LOADSCREENAZJOLNERUB76.BLP",
"LoadScreenAzjolUpperCity.blp",
"LoadScreenBaradinHold.blp",
"LoadScreenBlackrockCaverns.blp",
"LoadScreenBlackrockDepths.blp",
"LoadScreenBlackwingDescentRaid.blp",
"LoadScreenBlackWingLair.blp",
"LoadScreenBladesEdgeArena.blp",
"LoadScreenCave.blp",
"LOADSCREENCAVERNSTIME.BLP",
"LoadScreenChamberBlack.blp",
"LoadScreenChampionsHall.blp",
"LoadScreenDalaranPrison.blp",
"LoadScreenDalaranSewersArena.blp",
"LoadScreenDeadmines.blp",
"LOADSCREENDEATHKNIGHT.BLP",
"LoadScreenDeepholm.blp",
"LOADSCREENDEEPHOLMDUNGEON.BLP",
"LoadScreenDireMaul.blp",
"LoadScreenDrakTharon.blp",
"LoadScreenDungeon.blp",
"LoadScreenEnviroment.blp",
"LoadScreenGilneasBG.blp",
"LoadScreenGilneasBG2.blp",
"LoadScreenGnomeregan.blp",
"LOADSCREENGRIMBATOL.BLP",
"LoadScreenGrimBatolRaid.blp",
"LOADSCREENGUNDRAK.BLP",
"LoadScreenHallofLegends.blp",
"LoadScreenHallsofOrigination.blp",
"LoadScreenHallsofReflection.blp",
"LOADSCREENHELLFIRECITADEL5MAN.BLP",
"LOADSCREENHELLFIRECITADELRAID.BLP",
"LoadScreenHyjal.blp",
"LoadScreenIcecrown5man.blp",
"LoadScreenIcecrownCitadel.blp",
"LOADSCREENISLEOFCONQUEST.BLP",
"LoadScreenKarazhan.blp",
"LOADSCREENMALYGOS.BLP",
"LoadScreenMaraudon.blp",
"LoadScreenMoltenCore.blp",
"LoadScreenMonastery.blp",
"LoadScreenNagrandArenaBattlegrounds.blp",
"LoadScreenNexus70.blp",
"LoadScreenNexus80.blp",
"LoadScreenNorthrendBG.blp",
"LoadScreenOldStrathome.blp",
"LoadScreenOrgrimmarArena.blp",
"LoadScreenPitofSaron.blp",
"LoadScreenRagefireChasm.blp",
"LoadScreenRazorfenDowns.blp",
"LoadScreenRazorfenKraul.blp",
"LoadScreenRubySanctum.blp",
"LoadScreenRuinedCity.blp",
"LoadScreenRuinsofLordaeronBattlegrounds.blp",
"LoadScreenScholomance.blp",
"LOADSCREENSHADOWFANGKEEP.BLP",
"LoadScreenSkywall.blp",
"LoadScreenSkywallRaid.blp",
"LoadScreenStormwindStockade.blp",
"LoadScreenStrathome.blp",
"LoadScreenSunkenTemple.blp",
"LoadScreenSunwell5Man.blp",
"LOADSCREENTEMPESTKEEP.BLP",
"LoadScreenThroneoftheTides.blp",
"LoadScreenTolBarad.blp",
"LoadScreenTwinPeaksBG.blp",
"LOADSCREENULDAMAN.BLP",
"LoadScreenUlduar77.blp",
"LoadScreenUlduar80.blp",
"LOADSCREENULDUARRAID.BLP",
"LoadScreenUldumDungeon.blp",
"LOADSCREENUTGARDE.BLP",
"LoadScreenUtgardePinnecle.blp",
"LoadScreenWailingCaverns.blp",
"LoadScreenWintergrasp.blp",
"LOADSCREENZULAMAN.BLP",
"LoadScreenZulFarrak.blp",
"LoadScreenZulGurub.blp"
}